 /****************************************************\
|   Drift DriveWriter 1.0 | Revision Date: 4/01/2005   |
|    Copyright 2005-2020 Drift Solutions / Indy Sams   |
|      www.driftsolutions.com / www.titus-dev.com      |
 \****************************************************/

/*
Copyright (c) 2005-2020 Drift Solutions

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <math.h> // for ceil()

int main(int argc, char * argv[]) {
	char file[MAX_PATH];
	printf("What file do you want to write: ");
	gets(file);
	if (!strlen(file)) { printf("No file given, aborting...\n"); return 0; }
	printf("To what drive? [a,b,c,...]\n");
	int drive = toupper(getch());
	switch(drive) {
		case 'A':
			break;
		case 'B':
			break;
		default:
			if (drive >= 'C' && drive <= 'Z') {
				printf("Warning! Drive %c: is (probably) not a floppy!\nContinue anyway? [y/n]\n",drive);
				if (toupper(getch()) != 'Y') {
					return 0;
				} 
			} else {
				return 0;
			}
			break;
	}

	HANDLE hFile = CreateFile(file,GENERIC_READ|GENERIC_WRITE,NULL,NULL,OPEN_EXISTING,NULL,NULL);
	if (hFile == INVALID_HANDLE_VALUE) {
		printf("Error opening file for input!\n");
		return 0;
	}

	char buf[MAX_PATH];
	DWORD sectors_per_cluster,bytes_per_sector,free_clusters,num_clusters;
	wsprintf(buf,"%c:\\",drive);
	if (!GetDiskFreeSpace(buf,&sectors_per_cluster,&bytes_per_sector,&free_clusters,&num_clusters)) {
		printf("Error getting disk geometry! (Windows Error Code: %d)\n",GetLastError());
		printf("Assuming 512 sector size, 2880 sectors (standard 1.44mb floppy)\n");
		sectors_per_cluster = 1;
		bytes_per_sector    = 512;
		free_clusters       = 2880;
		num_clusters        = 2880;
	}

	wsprintf(buf,"\\\\.\\%c:",drive);
	HANDLE hDrive = CreateFile(buf,GENERIC_READ|GENERIC_WRITE,NULL,NULL,OPEN_EXISTING,NULL,NULL);
	if (hDrive == INVALID_HANDLE_VALUE) {
		// failed to open for exclusive access (occured also if the floppy is unformatted for some reason)
		// try again but with shared read/write
		hDrive = CreateFile(buf,GENERIC_READ|GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,NULL,NULL);
	}
	if (hDrive == INVALID_HANDLE_VALUE) {
		printf("Error opening drive for output! (Windows Error Code: %d)\n",GetLastError());
		CloseHandle(hFile);
		return 0;
	}

	DWORD disk_size = (sectors_per_cluster * num_clusters * bytes_per_sector);	
	printf("Drive %c: geometry -> Sector Size: %d (%d sectors), Disk Size: %d\n",drive,bytes_per_sector,disk_size / bytes_per_sector,disk_size);

	printf("Hit y to begin writing, anything else to abort...\n");
	if (toupper(getch()) != 'Y') {
		CloseHandle(hFile);
		CloseHandle(hDrive);
		return 0;
	}

	char * sect = new char[bytes_per_sector];
	DWORD bread=0;

	DWORD fSize = GetFileSize(hFile,NULL);
	DWORD nSectors = ceil((double)fSize / (double)bytes_per_sector);
	DWORD sCount=0;
	while (fSize > 0) {
		sCount++;
		printf("\rWriting sector %d/%d...",sCount,nSectors);

		if (fSize >= bytes_per_sector) {
			if (!ReadFile(hFile,sect,bytes_per_sector,&bread,NULL)) {
				printf("Error reading from source file! (Windows Error Code: %d)\n",GetLastError());				
			}
			if (!WriteFile(hDrive,sect,bytes_per_sector,&bread,NULL)) {
				printf("Error writing sector %d! (Windows Error Code: %d)\n",sCount,GetLastError());
			}
			fSize -= bytes_per_sector;
		} else {			
			memset(sect,0,bytes_per_sector);
			if (!ReadFile(hFile,sect,fSize,&bread,NULL)) {
				printf("Error reading from source file! (Windows Error Code: %d)\n",GetLastError());				
			}
			if (!WriteFile(hDrive,sect,bytes_per_sector,&bread,NULL)) {
				printf("Error writing sector %d! (Windows Error Code: %d)\n",sCount,GetLastError());
			}
			fSize -= fSize;
		}
	}
	printf("\nDone!\n");

	delete sect;
	return 0;
}
